/* =============================================================
   The functions required for the q-g initial-state 
   NNLO contributions to the DY rapidity distributions.
   ============================================================= */

#include "hardfns.hh"
#include "BornNLOfns.hh"

using namespace BornNLO;

// main function:

double qg(double y, double z){
  double u,t,r,Z2,z2,z3,z4,z5,z6,z7,z8,omz2,
        u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12, up12,up13,omu3,omu4,
        ln2,lnz,lnomz,lnopz,lnhfopz,lnu,lnumo,lnopu,lnhfopu,
        lnrpt,lnrmt,lnoptr,lnomtr,lnopt,lnomt,lnopr,lnrmo,lnomuz,lnumz,temp;
 u = (z+y*(1.-z))/(1.-y*(1.-z));   t = sqrt(z);   r = sqrt(u);
 Z2 = ZETA2;
 z2 = z*z;  z3 = z2*z;  z4 = z2*z2;  z5 = z3*z2;
 z6 = z3*z3;  z7 = z4*z3;  z8 = z4*z4;
 u2 = u*u;  u3 = u2*u;  u4 = u2*u2;  u5 = u3*u2;
 u6 = u3*u3;  u7 = u4*u3;  u8 = u4*u4;  u9 = u5*u4;
 u10 = u5*u5;  u11 = u6*u5;  u12 = u6*u6;
 up12 = (1.+u)*(1.+u);   up13 = up12*(1.+u);  
 omz2 = (1.-z)*(1.-z);   omu3 = (1.-u)*(1.-u)*(1.-u);  omu4 = omu3*(1.-u);
 ln2 = log(2.);  lnz = log(z);   lnomz = log(1.-z);  
 lnopz = log(1.+z);  lnhfopz = log((1.+z)/2.);
 lnu = log(u);   lnumo = log(abs(u-1.));
 lnopu = log(1.+u);  lnhfopu = log((1.+u)/2.);
 lnrpt = log(r+t);  lnrmt = log(r-t);
 lnoptr = log(1.+t*r);  lnomtr = log(1.-t*r);
 lnopt = log(1.+t);  lnomt = log(1.-t);  
 lnopr = log(1.+r);  lnrmo = log(abs(r-1.));  
 lnomuz = log(1.-u*z);  lnumz = log(u-z);
 temp =
// q^2 terms (mostly): 
// J's:
- 1./6. * z * (1-z) * (1+u)/u3 * ( 2*z2*u2+4*z2*u+2*z2+34*u2*z
 +16*u*z+9*u3*z+u2+9*u4 )
   * J2(1.-y,z)
- 3./2. * z * (1-z) * (1+u)/(u-z) * (1+2*z+z2+u2) 
   * J3(1.-y,z)
//
- 1./6. * z2  * (1-z) * ( 7*z5-1220*u7*z2+132*u2*z3+156*u2*z5
 -1182*u4*z4+290*z5*u3-88*u3*z2-2964*u5*z2+2038*z3*u5
 -1200*u4*z2-1712*u6*z2+1830*u6*z3-1280*u5*z4+49*z5*u+2720*u6*z
 -58*z4*u-1088*u7+237*u5*z5+94*u6*z5+16*u7*z5+600*u5*z+1726*u4*z3
 +856*u7*z-188*u7*z4-766*u6*z4+766*u7*z3+335*z5*u4-284*z4*u2
 -706*u3*z4+884*u3*z3)/u2/(u-z)/(z*up12-4*u2)/(z*up12-4*u2)
   * ( J21(y,z) 
     - (1+z) * (1-u*z)/(1+u)/r/(r+t)/(1+t*r)/omz2
         * lnz * lnhfopz
// borrowed from q^1 terms:
              + 1./z/(1.+u) * ( (lnrpt-lnopu) + 2.*u/(1.-u) * lnz 
              - 1./2. * (1.+u)/(1.-u) * (1.+t/r) * lnu )
       ) 
// polylog2's (mostly):
+ 1./12. * z2 * (1-z)/(1+z)/u/(u-z) * ( 99*u*z+73*u+42*z2*u+88*u2*z
  +64*u2+40*z2*u2-10*z2-10*z )
   * li2(z)
- 1./3. * z2 * (1-z) * (1-2*z+2*z2) * (1+u)/(1+z)/(u-z)
   * ( 5 * li2((1.+z)/2.) - 5./2. * lnopz*lnopz + 8 * ln2 * lnz )
- 1./6. * z * (1-z)/(1+z)/(u-z)/up13 * ( 12*u3*z4+8*z4*u+16*z4*u2
 -16*z3-64*z3*u-42*u3*z3-16*u4*z3-90*u2*z3+13*z2*u+8*z2
 +28*z2*u2+55*u3*z2+8*u4*z2+14*z+50*u*z+90*u2*z+14*u4*z
 +84*u3*z+15*u3+16*u2+5*u )
   * li2(-z)
//
+ 1./24. * z * (1-z)/u3/(u-z) * ( -792*z3-107*u4+144*u3-27*u6
 -16*u4*z3+648*u3*z2+188*u3*z3-48*u3*z-576*u2*z+644*z2*u2
 -596*u2*z3-406*z3*u+1224*z2*u+16*u5*z2-171*u4*z2+16*z3*u5
 -16*u6*z2-16*u5*z+106*u4*z ) 
   * li2(-u)
//
- 1./24. * z * (1-z)/u2/(u-z) * ( 22*z3-73*u3-9*u5-137*u3*z2
    +110*u3*z-30*u2*z+52*z2*u2+40*u2*z3-28*z3*u+32*z2*u ) 
   * li2((1.-u)/2.)
//
- 1./24. * z * (1-z)/u2/(u-z) * ( 26*z3-75*u3-27*u5-123*u3*z2
        +42*u3*z-60*u2*z+112*z2*u2+44*u2*z3-20*z3*u+60*z2*u )
   * li2((1.-u)/(1.+u))
//
+ 1./4. * z * (1-z)/u2/(u-z) * ( 13*u3-38*u3*z+29*u3*z2-3*u5
  +12*u2*z3-12*z2*u+12*u2*z-12*z3*u+6*z3 )
   * ( li2(-r) + li2((r-1.)/r) )
//
- 1./6. * z * u * (1-z) * (1+z)/up13/(u-z) 
   * (z2+4*z2*u2+2*z2*u-2*z-8*u*z+3+2*u+2*u2)
     * ( li2((1.-u*z)/(1.+u)) + lnopz * lnomuz )
//
+ 1./6. * z2 * (1-z)/u/(u-z) * (-10*z+9*u+2*u*z) 
  * ( li2(-(t-1.)/(r+1.)) - li2(-r*(t-1.)/(r+1.))
    - li2(-(r-1.)/(1.+t*r)) + li2((r-1.)/(r+t))
      - lnopt * (lnrpt - lnoptr) )
//
+ 1./3. * z * (1-z)/u3/up13/(u-z) * ( -103*u6*z+99*z3-120*u7*z
   +29*u5*z+292*u4*z3-481*u4*z2+26*u6*z3+37*u6*z2+72*u2*z
   +237*u3*z+251*u4*z+15*u7*z2-535*z2*u2-18*u3-60*u4-86*u5
   -54*u6-28*u7-7*u8-2*u9+504*u3*z3+558*u2*z3-721*u3*z2
   +98*z3*u5-159*u5*z2-35*u8*z+360*z3*u-153*z2*u )
   * li2(-2.*t/(r-t))
//
+ 1./12. * z * (1-z)/u3/up13/(u-z) * ( -505*u6*z+396*z3-514*u7*z
   +63*u5*z+1148*u4*z3-2013*u4*z2+98*u6*z3+239*u6*z2+288*u2*z
   +963*u3*z+1013*u4*z+103*u7*z2-2154*z2*u2-72*u3-207*u4-239*u5
   -86*u6+4*u7+53*u8+19*u9+1996*u3*z3+2222*u2*z3-2956*u3*z2
   +382*z3*u5-607*u5*z2-140*u8*z+1438*z3*u-612*z2*u ) 
   * li2((r-t)/2./r)
//
- 1./6. * z * (1-z)/u3/up13/(u-z) * ( 198*z3-103*u4-36*u3-32*u6
 -106*u5+559*u4*z3-1454*u3*z2+1008*u3*z3+480*u3*z+144*u2*z
 -1070*z2*u2+1116*u2*z3+720*z3*u-306*z2*u+56*u6*z-402*u5*z2
 -1003*u4*z2+146*z3*u5-68*u6*z2+8*u6*z3+278*u5*z+579*u4*z )
   * li2(t*r)
//
- 1./6. * z * (1-z)/u2/up13/(u-z) * ( 4*u6*z+18*z3-4*u7*z+4*u7*z3
 +56*u5*z+66*u4*z3-66*u4*z2+8*u6*z3-12*u6*z2+18*u2*z+71*u3*z
 +92*u4*z-8*u7*z2-52*z2*u2-5*u3-22*u4-22*u5-8*u6+15*u3*z3
 -4*u8*z2-18*u2*z3-73*u3*z2+40*z3*u5-28*u5*z2+18*z3*u
 -16*z2*u )
   * li2(-t*r)
//
- 1./6. * z * (1-z) * (1+z) * u/up13/(u-z) 
   * (1-2*u*z+u2+2*u2*z+2*z2*u2)
   * ( li2((1.-t*r)/2.) - ln2 * lnoptr ) 
//
- 1./12. * z * (1-z)/u3/up13/(u-z) * ( 577*u6*z-396*z3+526*u7*z
 +16*u7*z3+101*u5*z-1232*u4*z3+1853*u4*z2-166*u6*z3-221*u6*z2
 -288*u2*z-951*u3*z-919*u4*z-77*u7*z2+2142*z2*u2+72*u3+211*u4
 +257*u5+126*u6+68*u7+u8-u9-2018*u3*z3-16*u8*z2-2260*u2*z3
 +2864*u3*z2-526*z3*u5+541*u5*z2-8*u9*z2+8*u8*z3+132*u8*z
 -1460*z3*u+612*z2*u )
   * li2(t/r)
//
+ 1./12. * z * (1-z)/u3/up13/(u-z) * ( 613*u6*z-792*z3+546*u7*z
 -319*u5*z-1994*u4*z3+4303*u4*z2+2*u6*z3-51*u6*z2-576*u2*z
 -1839*u3*z-1903*u4*z-135*u7*z2+4302*z2*u2+144*u3+421*u4+443*u5
 +142*u6-20*u7-53*u8-19*u9-3832*u3*z3-4346*u2*z3+5992*u3*z2
 -418*z3*u5+1587*u5*z2+140*u8*z-2842*z3*u+1224*z2*u )
   * li2(-t/r)
//
+ 1./3. * z * (1-z)/u3/up13 * ( -54*u*z+99*z2+27*u6+59*u5+91*u4
 +63*u3+18*u2+2*u8+7*u7+288*u4*z2+23*u4*z+137*u5*z-165*u3*z
 +504*u3*z2-175*u2*z+558*z2*u2+360*z2*u+35*u7*z+123*u6*z
 +90*u5*z2+18*u6*z2 )
   * li2((1.-t*r)/(1.+u))
//
+ 1./6. * z * (1-z)/u2/up13/(u-z) * ( 9*z3-62*z2*u2-18*z2*u
 -2*u2*z+28*u4*z-5*u3*z3+8*u4*z3+34*u3*z+8*u7*z2+4*u8*z2
 -18*z3*u5-47*u3*z2+9*z3*u+19*u3+8*u2+27*u4+20*u5-8*u*z
 +40*u6+27*u7+9*u8-4*u7*z3-9*u2*z3+29*u4*z2-8*u6*z3
 +13*u6*z2-13*u5*z2+10*u6*z-56*u5*z+12*u7*z )
   * li2(r*(r-t)/(1.+u))
//
+ 1./6. * z * (1-z)/u2/(u-z) * ( 16*u3*z-8*u3-16*u3*z2+18*u2*z3
 -18*z2*u+18*u2*z-18*z3*u+9*z3 )
   * li2((1.-u)/(1.+t*r))
//
+ 1./6. * z * (1-z) * (1+z) * u/up13/(u-z) * ( 9*z2+16*z2*u2
 +18*z2*u-18*z-2*u2*z-34*u*z+8+18*u+17*u2 )
   * li2((u-z)/(1.+u))
//
- 8./3. * t * r * z * (1-z) * (1-2*z+2*z2)/(u-z)
    * ( li2((1.-u)/2.) + li2((1.-u)/(1.+u)) - li2(-u)
      + 2 * li2(-r) + 2 * li2((r-1.)/r) + 1./4. * lnu*lnu
      - lnu * lnopu - ln2 * lnopu + lnu * lnopr
      + 1./2. * lnopu*lnopu + 1./2. * ln2*ln2 + 1./2. * Z2 )
//
+ 1./3. * t/r * z * (1-z) * (1-2*u+2*u*z) * (u+z)/(u-z) 
   * ( - li2(-2.*t/(r-t)) - 4 * li2(t*r) + 4 * li2(-t*r)
       - 4 * li2((1.-t*r)/2.) + 4 * ln2 * lnomtr
       - 5 * li2((r-t)/2./r) - 5 * li2(t/r) + 5 * li2(-t/r)
       + 5 * ln2 * lnrmt - 5./8. * lnu*lnu
       + 5./2. * (lnrmt - ln2) * lnu
       - 2 * li2((1.-t*r)/(1.+u)) - 2 * li2(r*(r-t)/(1.+u)) 
       + 2 * (lnz - lnrmt - lnomtr) 
           * (lnrpt - lnrmt + lnoptr - lnomtr)
       + lnu * (lnoptr - 2 * lnomtr)
       - 3./2. * lnrmt*lnrmt - 2 * lnrmt * lnomtr 
       - lnomtr*lnomtr - lnoptr*lnoptr - lnrpt*lnrpt
       + 2 * lnopu * (lnrpt + lnoptr - lnopu)
       + lnu * lnopu - 1./2. * ln2*ln2 + 21./2. * Z2 )
// most of the ln^2's:
- 1./6. * z2 * (1-z) * (1-2*z+2*z2) * (1+u)/(1+z)/(u-z) * lnomz
       * ( 17. * ( lnomz + 2. * (lnumz - lnopu) ) + 26. * lnhfopz )
- 1./12. * z * (1-z)/u2
  * ( 2*z2*u2+4*z2*u+2*z2+34*u2*z+16*u*z+9*u3*z+u2+9*u4 )
  * ( lnumo * lnhfopu - 2 * lnu * lnrmo )
//
- 1./48. * z * (1-z)/(1+z)/u3/up13/(u-z) * (
 -309*u6*z+1188*z3+1188*z4-216*u7*z+48*u7*z4
 +38*u6*z4+408*u7*z3+553*u5*z-1140*u4*z3-6751*u4*z2+1168*u6*z3
 -4223*u6*z2-24*u8*z3-24*u9*z3+864*u2*z+2399*u3*z+2269*u4*z
 -24*u9*z2+806*u5*z4+3184*u4*z4+24*u8*z4-1152*u7*z2
 -5498*z2*u2-216*u3-584*u4-596*u5-236*u6-24*u7+6654*z4*u2
 -2316*u3*z3+5892*u3*z4-104*u8*z2+4326*z4*u+292*u2*z3
 -6913*u3*z2+894*z3*u5-6683*u5*z2-56*u8*z+2490*z3*u-1836*z2*u )
   * lnz*lnz
+ 1./12. * z2 * (1-z)/(1+z)/u/(u-z) * ( 240*z2*u2-10*z2+242*z2*u
  -10*z-229*u*z-240*u2*z+120*u2+129*u )
   * lnz * lnomz
//
+ 1./24. * z * (1-z)/u3/up13/(u-z) * ( 288*u2*z+924*u3*z
 +396*z3+1778*u3*z3+1418*z3*u-3232*u3*z2-1105*u5*z2
 +176*z3*u5-2597*u4*z2+75*u6*z2+36*u6*z3-430*u6*z
 +70*u5*z-2192*z2*u2-612*z2*u+101*u7*z2+2122*u2*z3
 -72*u3-233*u4-317*u5-176*u6-64*u7-u8+u9-8*u9*z2+8*u8*z3
 -16*u8*z2+948*u4*z-438*u7*z+16*u7*z3-124*u8*z+868*u4*z3 )
   * lnz * lnu
//
+ 1./24. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( 396*z3+396*z4
 -1854*z2*u2-612*z2*u+288*u2*z+1008*u4*z+2260*z4*u2-582*u3*z3
 +2018*u3*z4+335*u4*z3+915*u3*z-653*u7*z2-32*u8*z2-5*u8*z
 +1285*z3*u5-2421*u3*z2+848*z3*u-72*u3-235*u4-241*u5-94*u6
 +28*u7+27*u8+9*u9+9*u9*z-2*u6*z4+249*u7*z3+118*u2*z3
 +1140*u4*z4-2794*u4*z2+847*u6*z3-2396*u6*z2-3234*u5*z2
 +342*u5*z4-107*u6*z+1460*z4*u+430*u5*z-66*u7*z )
   * lnz * lnrmt
//
- 1./12. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( 396*z3+396*z4
 -1866*z2*u2-612*z2*u+288*u2*z+716*u4*z+2222*z4*u2-1024*u3*z3
 +1996*u3*z4-1171*u4*z3+859*u3*z-329*u7*z2-140*u8*z2-141*u8*z
 -763*z3*u5-1929*u3*z2+826*z3*u-72*u3-239*u4-341*u5-204*u6
 -84*u7-u8+u9+u9*z+102*u6*z4-11*u7*z3+68*u2*z3+1148*u4*z4
 -724*u4*z2-57*u6*z3+58*u6*z2-114*u5*z2+382*u5*z4-627*u6*z
 +1438*z4*u-270*u5*z-562*u7*z )
   * ln2 * lnrmt
//
+ 1./12. * z * (1-z)/u2/up13/(u-z) * ( 11*u2*z-43*u3*z-9*z3
 -2*u3*z3-9*z3*u+55*u3*z2+102*u5*z2+8*z3*u5+74*u4*z2
 +16*u6*z2+16*u6*z3-56*u6*z-52*u5*z+34*z2*u2+8*z2*u
 -16*u7*z2+9*u2*z3+8*u*z-10*u3-8*u2+18*u4+35*u5+4*u6
 -8*u8*z2-91*u4*z-16*u7*z+8*u7*z3-22*u4*z3 )
   * lnu * lnoptr
//
+ 2./3. * z * (1-z)/u/up13 * ( u6*z2+z2*u2+2*u3*z2
 +2*u5*z2+4*u4*z2+u3*z+7*u4*z+3*u2*z+3*u5*z+2*u*z+5*u2
 +5*u+2+u4+u3 )
   * lnomtr * lnoptr
//
+ 1./6. * z * (1-z)/up13/(u-z) * ( 36*u3*z3+34*u2*z3+8*u5*z
 +22*u+47*u3+46*u2+4*u4-40*u2*z-12*u*z+4*z+8*u4*z2+16*u4*z
 +54*u3*z2+41*u3*z-36*z2*u2-33*z2*u+17*z3*u-8*z2 )
   * lnrmt * lnomtr
//
+ 1./6. * z * (1-z)/u/up13 * ( 4*u6*z2+12*u5*z+8*u5*z2
 +16*u4*z2+28*u4*z+4*u4+3*u3+5*u3*z+10*u3*z2+18*u2+11*u2*z
 +5*z2*u2+20*u+8*u*z+8 ) 
   * lnrmt * lnoptr
//
+ 1./12. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -792*z3-792*z4
 +3716*z2*u2+1224*z2*u-592*u2*z-1735*u4*z-4437*z4*u2
 +1883*u3*z3-3915*u3*z4+1489*u4*z3-1797*u3*z+732*u7*z2
 +16*u8*z3+16*u9*z2+196*u8*z2+192*u8*z-16*u8*z4+190*z3*u5
 +4245*u3*z2-1665*z3*u+160*u3+516*u4+644*u5+340*u6+124*u7
 +28*u8+8*u9+8*u9*z-122*u6*z4-212*u7*z3-129*u2*z3-2169*u4*z4
 +3023*u4*z2-580*u6*z3+1413*u6*z2+2443*u5*z2+16*u9*z3
 -648*u5*z4+595*u6*z-2889*z4*u-175*u5*z+632*u7*z-32*u7*z4 )
  * lnz * lnopu
//
+ 1./6. * z * (1-z)/(1+z)/up13/(u-z) * ( -16*z4*u2-12*u3*z4
 -8*z4*u+106*u3*z3+128*z3*u+32*z3+186*u2*z3+32*u4*z3-32*z2
 -172*z2*u2-32*u4*z2-109*z2*u-151*u3*z2+90*u2*z+16*z+70*u*z
 +16*u4*z+36*u3*z-15*u3-5*u-16*u2 )
  * lnopz * lnopu
//
- 1./24. * z * (1-z)/(1+z)/u2/(u-z) * ( 20*z4*u-44*z4*u2-26*z4
 +251*u3*z3-28*u2*z3-40*z3*u-26*z3-180*z2*u2-47*u3*z2-60*z2*u
 +97*u3*z+124*u2*z+27*u5*z+75*u3+27*u5 )
  * ln2 * lnopu
//
- 1./6. * z * (1-z)/(1+z)/up13/(u-z) * ( 12*u3*z4+16*z4*u2+8*z4*u
 -48*u4*z3-170*u3*z3-282*u2*z3-192*z3*u-48*z3+40*u4*z2
 +183*u3*z2+220*z2*u2+141*z2*u+40*z2-2*u4*z+20*u3*z-6*u2*z
 -14*u*z-2*z+15*u3+16*u2+5*u )
  * lnz * lnopz
- 1./6. * z * (1-z)/(1+z)/up13/(u-z) * ( -18*z4*u2-16*u3*z4
 -9*z4*u+98*u3*z3+128*z3*u+32*z3+190*u2*z3+32*u4*z3-32*z2
 -160*z2*u2-32*u4*z2-109*z2*u-157*u3*z2+94*u2*z+16*z+66*u*z
 +16*u4*z+32*u3*z-17*u3-8*u-18*u2 )
  * lnopz * lnumz
//
+ 1./96. * z * (1-z)/u2/up13/(u-z) * ( -153*u2*z-327*u3*z+60*z3
 +234*u3*z3+84*z3*u+674*u3*z2+1264*u5*z2+370*z3*u5
 +1194*u4*z2+258*u6*z2+32*u6*z3-1004*u6*z-1061*u5*z
 +260*z2*u2+86*z2*u-32*u7*z2+6*u2*z3+190*u3+202*u4+36*u5
 -100*u6-2*u7+2*u8-16*u8*z2-799*u4*z-272*u7*z+16*u7*z3
 +486*u4*z3 ) 
  * lnu*lnu
//
+ 1./24. * z * (1-z)/u3/up13/(u-z) * ( 19*u9+53*u8+4*u7-86*u6
 -239*u5-207*u4-72*u3-607*u5*z2+396*z3-2154*z2*u2-612*z2*u
 +288*u2*z+963*u3*z+103*u7*z2+2222*u2*z3-2956*u3*z2+382*z3*u5
 -2013*u4*z2+239*u6*z2+98*u6*z3-505*u6*z+63*u5*z+1148*u4*z3
 -514*u7*z+1996*u3*z3+1438*z3*u+1013*u4*z-140*u8*z )
  * ln2 * lnu
//
+ 1./24. * z * (1-z)/u2/(u-z) * ( 10*z3+43*u3+27*u5+59*u3*z2
 +22*u3*z+114*u2*z-116*z2*u2+28*u2*z3-52*z3*u-112*z2*u )
   * lnu * lnopr
//
- 1./24. * z * (1-z)/u3/up13/(u-z) * ( 29*u9+79*u8-56*u7
 -254*u6-409*u5-251*u4-72*u3-797*u5*z2+396*z3-2198*z2*u2
 -612*z2*u+288*u2*z+1011*u3*z+141*u7*z2+2220*u2*z3-3152*u3*z2
 +502*z3*u5-2333*u4*z2+357*u6*z2+198*u6*z3-689*u6*z+71*u5*z
 +1160*u4*z3-846*u7*z+1938*u3*z3+1452*z3*u+1139*u4*z-256*u8*z ) 
   * lnu * lnrmt
//
- 1./12. * z * (1-z)/u3/up13/(u-z) * ( 8*u7+5*u6-66*u5-92*u4
 -36*u3-548*u5*z2+198*z3-1090*z2*u2-306*z2*u+144*u2*z+456*u3*z
 +16*u7*z2+1080*u2*z3-1570*u3*z2+126*z3*u5-1223*u4*z2
 -74*u6*z2+16*u6*z3-117*u6*z+44*u5*z+459*u4*z3-52*u7*z
 +900*u3*z3+720*z3*u+442*u4*z )
   * lnu * lnrpt
//
- 1./24. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -1188*z3-1188*z4
 +5558*z2*u2+1836*z2*u-864*u2*z-2660*u4*z-6724*z4*u2
 +2366*u3*z3-6050*u3*z4+1373*u4*z3-2691*u3*z+1313*u7*z2
 +264*u8*z2+401*u8*z-489*z3*u5+6317*u3*z2-2504*z3*u+216*u3
 +755*u4+1061*u5+690*u6+348*u7+137*u8+43*u9+43*u9*z-214*u6*z4
 -317*u7*z3-302*u2*z3-3476*u4*z4+4666*u4*z2-1171*u6*z3
 +2668*u6*z2+4206*u5*z2-1126*u5*z4+1083*u6*z-4340*z4*u
 -218*u5*z+1170*u7*z )
   * lnz * lnrpt
//
- 1./6. * z * (1-z)/(1+z)/up13 * ( 2*u3*z3+103*z2*u+115*z2*u2
 +59*u3*z2+32*z2+9*u4*z2-32*z+36*u4*z-86*u*z-66*u2*z+9*u5*z
 +21*u3*z+27*u4+44*u3+16+59*u2+51*u+9*u5 )
   * ln2 * lnrpt
//
+ 1./3. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -99*z3-99*z4
 +463*z2*u2+153*z2*u-72*u2*z-241*u4*z-558*z4*u2+195*u3*z3
 -504*u3*z4+97*u4*z3-232*u3*z+123*u7*z2+33*u8*z2+40*u8*z
 -83*z3*u5+509*u3*z2-207*z3*u+18*u3+63*u4+91*u5+59*u6+27*u7
 +7*u8+2*u9+2*u9*z-18*u6*z4-41*u7*z3-23*u2*z3-288*u4*z4
 +321*u4*z2-147*u6*z3+173*u6*z2+259*u5*z2-90*u5*z4+121*u6*z
 -360*z4*u-17*u5*z+131*u7*z )
   * lnrmt * lnrpt
//
- 1./6. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -99*z3-99*z4
 +463*z2*u2+153*z2*u-72*u2*z-147*u4*z-558*z4*u2+245*u3*z3
 -504*u3*z4+293*u4*z3-209*u3*z+69*u7*z2+31*u8*z2+38*u8*z
 +205*z3*u5+462*u3*z2-207*z3*u+18*u3+54*u4+72*u5+40*u6+26*u7
 +7*u8+2*u9+2*u9*z-34*u6*z4+5*u7*z3-23*u2*z3-296*u4*z4
 +148*u4*z2+9*u6*z3-56*u6*z2+4*u5*z2-106*u5*z4+181*u6*z
 -360*z4*u+125*u5*z+150*u7*z )
   * lnrmt*lnrmt
//
+ 1./6. * z * (1-z)/(u-z)/up13 * ( 24*u3+24*u2+2*u4+11*u
 -20*u2*z-7*u*z+2*z+4*u4*z2+8*u4*z+26*u3*z2+20*u3*z-18*z2*u2
 -17*z2*u+9*z3*u+18*u3*z3+18*u2*z3+4*u5*z-4*z2 )
   * lnomtr*lnomtr
//
- 1./12. * z * (1-z)/up13/(u-z) * ( 100*u3+98*u2+8*u4+49*u
 -86*u2*z-25*u*z+8*z+12*u4*z2+44*u4*z+96*u3*z2+100*u3*z
 -98*z2*u2-73*z2*u+41*z3*u+80*u3*z3+82*u2*z3+20*u5*z
 -8*u4*z3+8*u5*z2-4*z3*u5+4*u6*z2-16*z2 )
   * lnz * lnomtr
//
- 1./12. * z * (1-z)/u3/up13/(u-z) * ( 144*u2*z+480*u3*z
 +198*z3+1008*u3*z3+720*z3*u-1454*u3*z2-312*u5*z2
 +190*z3*u5-983*u4*z2+130*u6*z2+60*u6*z3-204*u6*z
 +38*u5*z-1070*z2*u2-306*z2*u+46*u7*z2+1116*u2*z3-36*u3
 -109*u4-152*u5-86*u6-50*u7-14*u8-4*u9-4*u9*z2+4*u8*z3
 -8*u8*z2+511*u4*z-228*u7*z+8*u7*z3-62*u8*z+581*u4*z3 ) 
   * lnu * lnomtr
//
- 1./6. * z * (1-z)/u3/up13/(u-z) * ( -198*z3+1070*z2*u2
 +306*z2*u-152*u2*z-533*u4*z-1008*u3*z3-572*u4*z3-496*u3*z
 -30*u7*z2-4*u8*z3+4*u9*z2+8*u8*z2+78*u8*z-172*z3*u5
 +1438*u3*z2-720*z3*u+44*u3+153*u4+218*u5+139*u6+58*u7
 +14*u8+4*u9-8*u7*z3-1116*u2*z3+942*u4*z2-36*u6*z3
 -68*u6*z2+288*u5*z2+239*u6*z-78*u5*z+260*u7*z )
   * lnopu * lnomtr 
//
- 1./6. * z * (1-z)/u3/up13/(u-z) * ( 144*u2*z+476*u3*z+198*z3
 +1008*u3*z3+720*z3*u-1446*u3*z2-282*u5*z2+162*z3*u5
 -953*u4*z2+44*u6*z2+16*u6*z3-241*u6*z+92*u5*z-1070*z2*u2
 -306*z2*u+34*u7*z2+1116*u2*z3-36*u3-136*u4-200*u5-137*u6
 -54*u7-14*u8-4*u9+520*u4*z-232*u7*z-66*u8*z+567*u4*z3 )
   * lnrpt * lnomtr
//
- 1./12. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -396*z3-396*z4
 +1850*z2*u2+612*z2*u-304*u2*z-1028*u4*z-2222*z4*u2+856*u3*z3
 -1996*u3*z4+515*u4*z3-967*u3*z+529*u7*z2+8*u8*z3+8*u9*z2
 +172*u8*z2+157*u8*z-8*u8*z4-213*z3*u5+2037*u3*z2-826*z3*u
 +88*u3+307*u4+433*u5+264*u6+88*u7+u8-u9-u9*z-70*u6*z4
 -165*u7*z3-68*u2*z3-1124*u4*z4+1264*u4*z2-543*u6*z3
 +650*u6*z2+966*u5*z2+8*u9*z3-334*u5*z4+427*u6*z-1438*z4*u
 -162*u5*z+526*u7*z-16*u7*z4 )
   * lnrmt * lnopu
//
+ 1./12. * z * (1-z)/(1+z)/u3/(u-z) * ( 396*z3-53*u4-72*u3+396*z4
 -4*u5+19*u6+175*u4*z3-131*u3*z2-44*u3*z3+115*u3*z+34*u3*z4
 +288*u2*z-66*z2*u2+248*z4*u2-106*u2*z3-344*z3*u+268*z4*u
 -612*z2*u+19*u6*z-132*u5*z2-87*u4*z2-136*u5*z-75*u4*z )
   * lnrpt * lnopu
//
+ 1./24. * z * (1-z)/u2/up13/(u-z) * ( -16*u*z-27*u8-81*u7
   -148*u6-276*u5-217*u4-43*u3+16*u2-321*u5*z2+44*z3+168*z2*u2
   +24*z2*u-64*u2*z-102*u3*z+16*u7*z2+44*u2*z3+277*u3*z2
   +24*z3*u5+7*u4*z2-115*u6*z2-16*u6*z3+26*u6*z-146*u5*z
   +136*u4*z3+24*u7*z-8*u7*z3+92*u3*z3+76*z3*u-130*u4*z+8*u8*z2 )
   * lnu * lnopu
//
+ 1./48. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( -792*z3-792*z4
 +3712*z2*u2+1224*z2*u-608*u2*z-2055*u4*z-4538*z4*u2
 +1610*u3*z3-4018*u3*z4+683*u4*z3-1946*u3*z+1175*u7*z2
 +16*u8*z3+16*u9*z2+344*u8*z2+287*u8*z-16*u8*z4-713*z3*u5
 +3986*u3*z2-1714*z3*u+176*u3+741*u4+1191*u5+844*u6+188*u7
 -25*u8-11*u9-11*u9*z-64*u6*z4-243*u7*z3-218*u2*z3-2278*u4*z4
 +2085*u4*z2-669*u6*z3+1927*u6*z2+1571*u5*z2+16*u9*z3
 -716*u5*z4+1456*u6*z-2938*z4*u-121*u5*z+1094*u7*z-32*u7*z4 )
   * lnopu*lnopu
//
- 1./6. * z * (1-z)/u/up12/(u-z) * ( -8*z+8*u+4*u4+17*u3+19*u2
 -4*u4*z3-2*u3*z2+4*u3*z3+3*u3*z-9*u2*z-12*z2*u2+4*u2*z3
 -12*u*z-8*z2*u+4*u5*z2-4*z3*u5+4*u6*z2+12*u5*z+16*u4*z )
   * lnopu * lnoptr
//
- 1./12. * z * (1-z)/u/up13/(u-z) * ( -16*z+16*u+10*u4+42*u3
 +43*u2+8*u5-28*u4*z3-22*u3*z2-6*u3*z3+14*u3*z-23*u2*z
 -29*z2*u2-3*u2*z3-40*u*z-16*z2*u+28*u6*z+4*u5*z2-56*u4*z2
 -24*z3*u5+24*u6*z2-12*u6*z3+12*u7*z2+68*u5*z+14*u4*z )
   * lnz * lnoptr
//
- 1./6. * z * (1-z)/u2/up13/(u-z) * ( 18*u4*z3-9*u2*z3+9*z3*u
 +9*z3-16*u6*z2-66*u5*z2-48*u4*z2-61*u3*z2-54*z2*u2-18*z2*u
 +16*u6*z+48*u5*z+120*u4*z+79*u3*z+18*u2*z-42*u5-8*u6-17*u3-42*u4 )
   * lnrpt * lnoptr
//
+ 1./12. * z * (1-z)/u2/up13/(u-z) * ( 9*z3-34*z2*u2-8*z2*u
 -11*u2*z+83*u4*z+5*u3*z3+28*u4*z3+31*u3*z+8*u7*z2+4*u8*z2
 +2*z3*u5-58*u3*z2+9*z3*u+12*u3+8*u2-4*u4-20*u5-8*u*z-4*u6
 -4*u7*z3-9*u2*z3-62*u4*z2-8*u6*z3-12*u6*z2-70*u5*z2
 +44*u6*z+57*u5*z+12*u7*z )
   * lnoptr*lnoptr
//
- 1./24. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( 396*z3+396*z4
 -1846*z2*u2-612*z2*u+288*u2*z+990*u4*z+2222*z4*u2-816*u3*z3
 +1996*u3*z4-499*u4*z3+925*u3*z-465*u7*z2-132*u8*z2-133*u8*z
 +205*z3*u5-2055*u3*z2+826*z3*u-72*u3-251*u4-361*u5-224*u6
 -80*u7-u8+u9+u9*z+70*u6*z4+157*u7*z3+88*u2*z3+1132*u4*z4
 -1350*u4*z2+531*u6*z3-648*u6*z2-1060*u5*z2+350*u5*z4
 -373*u6*z+1438*z4*u+164*u5*z-462*u7*z )
   * lnrpt*lnrpt
//
+ 1./48. * z * (1-z)/(1+z)/u3/up13/(u-z) * ( 47*u9+133*u8+108*u7
 +52*u6-259*u5-345*u4-144*u3+148*u6*z4-609*u5*z2+792*z3+792*z4
 -3764*z2*u2-1224*z2*u+576*u2*z+1772*u3*z-715*u7*z2+66*u2*z3
 +2238*u4*z4-4024*u3*z2-791*z3*u5-1819*u4*z2-181*u6*z2
 +649*u6*z3+672*u5*z4-1430*u6*z+2854*z4*u-605*u5*z-2223*u4*z3
 -1070*u7*z+263*u7*z3+4406*z4*u2+3970*u3*z4-2170*u3*z3+1630*z3*u
 +1497*u4*z-147*u8*z-280*u8*z2+47*u9*z )
   * ln2*ln2
//
- 1./48. * z * (1-z)/(1+z)/up13/(u-z)/u3 * ( 119*u9+349*u8
   +380*u7+116*u6-267*u5-393*u4-144*u3-112*u6*z4+237*u5*z2
   +792*z3+792*z4-3824*z2*u2-1224*z2*u+576*u2*z+1774*u3*z
   -51*u7*z2-298*u2*z3+1322*u4*z4-4438*u3*z2-2531*z3*u5
   -2165*u4*z2+569*u6*z2-295*u6*z3+148*u5*z4-796*u6*z
   +2822*z4*u+153*u5*z-4579*u4*z3-358*u7*z-64*u7*z4+207*u7*z3
   +4102*z4*u2+3182*u3*z4-3590*u3*z3+1598*z3*u+1679*u4*z
   +165*u8*z-120*u8*z2+32*u9*z2+32*u8*z3+32*u9*z3-32*u8*z4
   +119*u9*z ) 
   * Z2
// rest of q^1 terms:
- 1./12. * z * (1-z) * (1+4*(1+u)*z) /u2/up12
 * ( u * (34+66*u+51*u2+18*u3) + 2. * (1+u) * (2+4*u+u2) * z ) 
     * J27(1.-y,z)
//
- 1./6. * z * (1-z) * (1+u)/(1+z)/(u-z) * ( 12-5*z+120*z2-133*z3 )
     * lnomz
- 2./3. * z * (1-z) * (1+u)/(1+z)/(u-z) * ( 15+16*z+35*z2-26*z3 )
     * lnhfopz
- 1./48. * z/(1+z)/u3/up13/(1-u)/(u-z)/(z*up12-4*u2) * (
  1896*z4-1896*z6-2692*u8*z2-16392*u7*z2-21132*u2*z3
 +21104*u2*z5+48*z6*u11+11222*u5*z6-29580*u4*z4+13450*z5*u3
 +23642*u3*z2+42398*u9*z3-23727*u5*z2+42860*z3*u5+28182*u4*z2
 -42920*u6*z2-39493*u6*z3+64173*u5*z4-16522*u10*z4+4032*z5*u
 +13261*u6*z+5424*z4*u-528*u3*z-6290*u8*z6+2232*z2*u2+38940*u8*z5
 +114*u11*z+2112*u5+2408*u6-4808*u7-10924*u8-6832*u9+292*u10+344*u11
 -83886*u5*z5-43412*u6*z5+20502*u7*z5-12826*u5*z+37306*u4*z3
 +28424*u7*z+42558*u7*z4+114446*u6*z4+82*u11*z4-1558*z6*u9
 +110*u11*z3+5648*u9*z+3196*u10*z5-4266*u6*z6-9826*u7*z6
 +112*u10*z6-304*u11*z5-11737*u9*z2+22650*u8*z+13801*u10*z3
 -49541*u9*z4+21598*u9*z5-50458*u8*z4+8766*u8*z3+48*u12*z3
 -394*u11*z2+3251*u10*z-48*z5*u12-192*z2*u12-4130*u10*z2
 -494*u2*z6+15010*u3*z6+22306*u4*z6-60386*u7*z3-47652*z5*u4
 -1702*z4*u2-34376*u3*z4-17174*u3*z3+192*z4*u12-10586*u4*z
 -5424*z6*u-4032*z3*u )
   * lnz
//
- 1./24. * z * (1-z)/u2/(1-u)/up13/(u-z)/(z*up12-4*u2) * ( 
  198*z3-382*z4+1569*u8*z2+10565*u7*z2+4947*u2*z3-216*z2*u
 +2201*u4*z4-5561*u3*z2-318*u9*z3+4781*u5*z2-8972*z3*u5
 -4577*u4*z2+11037*u6*z2-9921*u6*z3+3220*u5*z4+24*u10*z4
 -5440*u6*z-1526*z4*u+988*u3*z-1796*z2*u2-6*u11*z-216*u4-48*u5
 +968*u6+2128*u7+680*u8+648*u9+168*u10+24*u11+54*u2*z-496*u5*z
 -1017*u4*z3-4070*u7*z+1042*u7*z4+2246*u6*z4-24*u11*z3-864*u9*z
 +255*u9*z2-4428*u8*z-192*u10*z3+96*u9*z4+377*u8*z4-1151*u8*z3
 +96*u11*z2-210*u10*z+231*u10*z2-3979*u7*z3-2098*z4*u2
 -464*u3*z4+5895*u3*z3+2120*u4*z+1414*z3*u
+ 2. * t/r * ( -198*z3+7*z4-146*u8*z2-1386*u7*z2-1938*u2*z3
 +240*z2*u+625*u4*z4+4666*u3*z2-61*u9*z3+220*u5*z2-251*z3*u5
 +3906*u4*z2-1950*u6*z2-6*u6*z3+572*u5*z4+518*u6*z+56*z4*u
 -1394*u3*z+2064*z2*u2-53*u11*z+288*u4+1160*u5+440*u6-28*u7
 -152*u8+320*u9+128*u10+20*u11-72*u2*z-2615*u5*z-1042*u4*z3
 +1875*u7*z+110*u7*z4+331*u6*z4-453*u9*z+408*u9*z2+326*u8*z
 -12*u10*z3+16*u8*z4-84*u8*z3+12*u11*z2-386*u10*z+158*u10*z2
 -29*u7*z3+205*z4*u2+446*u3*z4-1963*u3*z3-3922*u4*z-976*z3*u ) 
 )   * lnu
//
+ 1./24. * z * (1-z)/(1+z)/u3/up13/(u-z)/(z*up12-4*u2) * (
 -1896*z4-1896*z5-1306*z2*u2+6716*u4*z+1426*z4*u2+50010*u3*z3
 +39022*u3*z4+29462*u4*z3+202*u3*z-17162*u7*z2
 +5739*u8*z3+435*u9*z2+378*u8*z2+10885*u8*z-2664*u8*z4
 -114*u10*z3-30516*z5*u3-14631*z3*u5-20502*u3*z2-338*u10*z
 -292*u10*z2+3240*z3*u-314*u8*z5-224*u9*z5+48*u11*z4
 -144*u11*z3-36*u11*z-808*u5-4384*u6-7640*u7-4932*u8-2220*u9
 -536*u10-48*u11+1223*u9*z-48*u10*z5+16262*u6*z4-21834*z5*u4
 -7503*u7*z3+24530*u2*z3+67400*u4*z4-24394*u2*z5+368*u10*z4
 -59832*u4*z2-22621*u6*z3-41352*u6*z2-180*u11*z2+128*u9*z4
 -70179*u5*z2+2568*u9*z3+51244*u5*z4+33197*u6*z-7276*z4*u
 -8136*u5*z5-10516*z5*u-698*u6*z5+24072*u5*z+21543*u7*z
 -3134*u7*z4+208*u7*z5 )
   * lnopu
//
+ 1./6. * z * (1-z)/(1+z)/u2/up13/(u-z) * ( -297*z3-99*z2
 -198*z4+488*z2*u2+50*z2*u+206*u2*z+158*u4*z-822*z4*u2
 +514*u3*z3-665*u3*z4+935*u4*z3+110*u3*z+497*z3*u5
 +296*u3*z2-675*z3*u-111*u3-39*u2-103*u4-80*u5+108*u*z+4*u7
 -4*u6*z4+4*u7*z3-293*u2*z3-340*u4*z4-100*u4*z2+175*u6*z3
 -19*u6*z2-346*u5*z2-123*u5*z4+72*u6*z-621*z4*u+38*u5*z )
   * lnrmt
//
+ 1./24. * z/(1+z)/u3/up13/(u-z)/(z*up12-4*u2) * ( 
  1896*z4-1896*z6+1368*z2*u2-7234*u4*z+23046*z4*u2
 -70392*u3*z3+10738*u3*z4-88916*u4*z3-264*u3*z+37233*u7*z2
 -6051*u8*z3+424*u9*z2+9979*u8*z2-14541*u8*z-10516*z6*u
 -24394*u2*z6+8769*u8*z4-828*u6*z6+106*u10*z3+69496*z5*u3
 -54694*z3*u5+20934*u3*z2-21758*u4*z6-30480*u3*z6-8210*u5*z6
 +422*u7*z6-106*u10*z+92*u8*z6-396*u10*z2-3240*z3*u
 -3236*u8*z5-620*u9*z5+12*u11*z3-12*u11*z+1056*u5+4968*u6
 +7744*u7+5012*u8+1956*u9+392*u10+48*u11-2889*u9*z-39031*u6*z4
 +89370*z5*u4-9531*u7*z3-25796*u2*z3-38128*u4*z4+25784*u2*z5
 +4*u10*z4+66674*u4*z2-18215*u6*z3+74375*u6*z2-48*u11*z2
 +3020*u9*z4+94321*u5*z2-1891*u9*z3-66103*u5*z4-37847*u6*z
 +10516*z4*u+59250*u5*z5+3240*z5*u+16538*u6*z5-25684*u5*z
 -29583*u7*z-2911*u7*z4-3342*u7*z5 )
  * lnrpt
//
+ 1./6. * z * (1-z)/(1+z)/u2/up13/(u-z) * ( -297*z3-99*z2
 -198*z4+615*z2*u2+54*z2*u+209*u2*z+135*u4*z-822*z4*u2
 -51*u3*z3-665*u3*z4+117*u4*z3+110*u3*z+2*u7*z2-41*z3*u5
 +801*u3*z2-675*z3*u-75*u3-27*u2-58*u4-47*u5+108*u*z+12*u6
 +4*u7-4*u6*z4+4*u7*z3-443*u2*z3-340*u4*z4+658*u4*z2
 +40*u6*z3+107*u6*z2+154*u5*z2-123*u5*z4+67*u6*z-621*z4*u
 +13*u5*z )
   * lnomtr
//
+ 1./12. * z/(1+z)/u/up13/(u-z) * ( -31*z+29*z3-20*z2+18*z4
 -13*z2*u2-20*z2*u-6*u2*z+141*u4*z-56*z4*u2-21*u3*z3
 +120*u3*z4-112*u4*z3-50*u3*z-24*u7*z2+59*z5*u3-4*z3*u5
 -113*u3*z2+67*z3*u+31*u+13*u3+73*u2+10*u4-33*u5-53*u*z
 -18*u6+64*u6*z4-51*z5*u4+4*u2*z3+19*u4*z4-2*u2*z5
 -3*u4*z2+13*u6*z3-42*u6*z2-29*u5*z2+78*u5*z4+7*u6*z
 -15*z4*u-64*u5*z5-18*z5*u-24*u6*z5+52*u5*z+24*u7*z4 )
   * lnoptr
//
+ 1./6. * t/r * z * (1-z)/up12/(u-z) * ( u3*z2-z2*u-2*z2*u2
 +28*u3*z+8*u2*z+6*z+16*u4*z+6*u*z-9*u5-6*u-18*u4-15*u3-14*u2 )
   * ( - lnz + lnoptr + lnrpt - 2. * ln2 )
//
+ 1./3. * t/r * z * (1-z)/u2/up12/(u-z) * ( -2*u7*z-2*u7
 +2*u6*z2-u6-39*u6*z+15*u5*z2+27*u5-86*u5*z-172*u4*z
 +70*u4*z2+59*u4+205*u3*z2+82*u3-274*u3*z+297*z2*u2
 -297*u2*z+36*u2-120*u*z+261*z2*u+99*z2 )
   * ( - lnz + lnomtr + lnrmt )
// q^0 terms:
- 1./24. * z * (1-z)/(1+z)/u2/up13/(u-z)/(z*up12-4*u2) * (
 2 * t/r * (1+u) * (1+z) * ( 552*z3+1961*u3*z+216*u4+348*u5-784*u6
  -1092*u7-360*u8-88*u9+16*u8*z3+7858*u5*z+2037*u4*z3+2955*u7*z
  -6205*u5*z2-11769*u4*z2+573*z3*u5-2958*u6*z2+358*u6*z3
  -1732*u7*z2+5001*u2*z3+5796*u6*z+4557*u3*z3-11629*u3*z2
  -538*z2*u-4813*z2*u2+6072*u4*z+2658*z3*u+176*u7*z3-54*u2*z
  +86*u9*z-16*u9*z2+1174*u8*z-356*u8*z2 )
- ( 1146*z3+2250*z4+1104*z5+48*u8*z4+3902*u3*z+296*u4+2232*u5
  +2948*u6+1812*u7+192*u8-176*u9-2057*u8*z3+8141*u6*z4-3431*u5*z5
  +3130*u7*z4-2125*u6*z5-356*u7*z5+10963*u5*z-39769*u4*z3
  -497*u7*z+24986*u5*z2+2117*u4*z2-26690*z3*u5+26376*u6*z2
  +4168*u5*z4-17919*u6*z3+15164*u7*z2+9634*u2*z5-8801*u2*z3
  +5274*z5*u+9882*z4*u+2753*u6*z+12183*z4*u2-34549*u3*z3+688*u3*z4
  -4578*u4*z4-15426*u3*z2+7485*z5*u3-1152*z2*u-10020*z2*u2
  +10601*u4*z+3456*z3*u-9929*u7*z3+327*z5*u4-74*u2*z-228*u9*z
  +100*u9*z2-228*u8*z+32*u8*z5-32*u9*z4+120*u9*z3+4527*u8*z2 ) ) ;
 return temp ; 
}

// u -> 1 limit of qg:

double qg_u1(double z){
 double t,Z2,z2,z3,z4,z5, ln2,lnz,ln1mz,tp12,ln1pt,ln1pz, temp;
 t = sqrt(z);   Z2 = ZETA2;  
 ln2 = log(2.);  lnz = log(z);  ln1mz = log(1.-z);  
 tp12 = (1.+t)*(1.+t);  ln1pt = log(1.+t);  ln1pz = log(1.+z);
 z2 = z*z;  z3 = z2*z;  z4 = z2*z2;  z5 = z3*z2;
 temp = z/(1+z) * ( 
- 1./3. * (1-z) * (1+z) * (8*z2+59*z+10) * J2_u1(z)
- 3. * (1+z) * (2+2*z+z2) * J3_u1(z)
+ 1./6. * z * (1+z) * (1-z) * (74*z2-57*z+68) * J21_u1(z)
//
- 1./48. * ( -2374*z2-472*z-348*z3+1937*z4-24*z2*t-16*z3*t+8*t-255 )
   * li2(z)
- 1./12. * ( 9*z4-57*z3+28*z2+63*z+9 ) * li2(-z)
- 1./12. * ( -84*z2+47*z+9+87*z3+9*z4 ) * li2((1.+z)/2.)
+ 1./24. * (1+z) * ( -45+131*z3+229*z2-563*z+72*t-72*t*z-144*z2*t )
   * li2(t)
+ 1./24. * (1+z) * ( -217+243*z3+441*z2-291*z-56*t+56*t*z+112*z2*t )
   * li2(-t)
- 1./12. * (1+z) * ( -232+972*z3-973*z2+187*z-48*t+48*t*z+96*z2*t )
   * li2((1.-t)/2.)
//
- 1./48. * (-207+367*z3-4148*z2+688*z+2770*z4) * lnz*lnz
+ 1./12. * ( 675*z3+364*z-1353*z2-68+452*z4+16*t-48*z2*t-32*z3*t ) 
   * lnz * ln1mz
- 1./12./(1+t)/(1+t) * ( 83*z5+18*t*z4-85*z4-370*z3*t-z3+312*z2*t
   +234*z2-2*t*z+76*z+18*t+9 )
   * lnz * ln1pz
+ 1./6. * (1+z) * ( 501*z3+32*z2*t-523*z2+16*t*z+124*z-16*t-75 )
   * lnz * ln1pt
- 1./6. * (1+z) * ( 1001*z3+64*z2*t-997*z2+32*t*z+179*z-32*t-99 )
   * ln1mz * ln1pt
- 1./12. * (1-t) * ( 38*z3*t+70*z3-403*z2*t-355*z2+151*t*z+151*z-38*t-54 )
   * ln1mz*ln1mz
- 1./3. * z * (-2*z+2*z2+1) * ln1pz * ( 42*ln1mz - 5*ln1pz - 16*ln2 )
//
- 1./24./(1+t)/(1+t) * ( 3607*z5+7510*t*z4+3775*z4+404*z3*t
  -4362*z3-9016*z2*t-3820*z2+1692*t*z+255*z-910*t-455 )
   * ln2 * lnz
+ 1./6. * ( -1220*z2+217*z-153+963*z4+373*z3-16*t+48*z2*t+32*z3*t ) 
   * ln2 * ln1mz
- 1./12. * (1+z) * (970*z3+32*z2*t-875*z2+16*t*z+49*z-16*t-130)
    * ln2 * ln1pt
+ 1./24. * (1+z) * (2972*z3+160*z2*t-2869*z2+80*t*z+407*z-80*t-328)
   * ln1pt*ln1pt
//
- 1./24. * (-16*t+48*z2*t+32*z3*t-176+379*z3-1342*z2+379*z+956*z4)
    * ln2*ln2
+ 1./48. * ( 170*z3+2023*z4+480*z2*t+320*z3*t-2066*z2+310*z-160*t-309 )
   * Z2
// q^1:
- 1./48. * (1-z) * (1+z) * (1+8*z) * (169+28*z) * J27_u1(z)
+ 1./24. * (1-t) * ( 2773*z3*t-1023*z3-1844*z2*t-1680*z2-2001*t*z
   +1155*z+448*t-356 ) * ln1mz
- 1./96. * ( 11704*z4-16*z3*t-4036*z3+496*z2*t-21069*z2+16*t*z
  +7108*z-496*t-3947 ) * ln2
+ 1./96. * (1+z) * ( 22796*z3-15200*z2*t-29188*z2+16352*t*z
   +7475*z-3712*t-987 ) * ln1pt
+ 4./3. * (-35*z2+26*z3-16*z-15) * ln1pz
+ 1./192. * ( 10240*z4-15168*z3*t-7668*z3+160*z2*t+5707*z2+12608*t*z
   -7900*z-2720*t+1445 ) * lnz
// q^0:
- 1./96. * (1-t) * ( 2243*z3*t-5721*z3+1007*z2*t+5087*z2
   -5059*t*z+6105*z+1793*t+913 ) );
 return temp;
}

// u -> z limit of qg:

double qg_uz(double y, double z){
 double u,r,t,Z2,z2,z3,z4,z5,z6, ln2,lnz,ln1mz,ln1pz,lnhf1pz, lnde,  temp;
 u = (z+y*(1.-z))/(1.-y*(1.-z));   t = sqrt(z);   r = sqrt(u);
 Z2 = ZETA2;  z2 = z*z;   z3 = z2*z;  
 z4 = z2*z2;  z5 = z3*z2;  z6 = z3*z3;
 ln2 = log(2.);  lnz = log(z);  ln1mz = log(1.-z); ln1pz = log(1.+z);
 lnhf1pz = log((1.+z)/2.);
 lnde = log(r-t);
 temp = 1./(1+z) * ( 
// q^2:
- 1./3. * (1-z) * lnde * ( 
  + z * (2*z3-24*z2+15*z-3) * lnde
  -  z * (53*z3-35*z2+66*z+4) * lnz 
  + z * ( 14*z3-79*z2+46*z-11 ) * ln1mz 
  + ( 55*z4+118*z3+165*z2+122*z+63 ) * lnhf1pz )
//
- 1./6. * z * (1-z) * (1+z) * (20*z2+19+38*z) * J2rem(z)
- 1./2. * z * (1-z) * ( 42*z3+34*z2+37*z+13 ) * li2(z)
+ 1./3. * (1-z) * ( 222*z+291*z2+156*z3+55*z4+126 ) * li2(-z)
+ 1./12. * (1-z) * ( 757*z2+493*z+586*z3+200*z4+252 ) * li2((1.+z)/2.)
//
- 1./6. * z * (1-z) * (3+62*z-62*z2+93*z3) * lnz*lnz
+ 1./6. * z * (1-z) * ( 26*z3-189*z2+114*z-25 ) * ln1mz * lnz
+ 1./12./(1-z) * ( -241*z4-883*z3-267*z2-71*z+630+306*z5+422*z6 )
   * ln1pz * lnz
- 1./12./(1-z) * ( 122*z6+50*z5-221*z4+81*z3-235*z2-27*z+126 )
   * ln2 * lnz
- 1./6. * z * (1-z) * ( 14*z3-78*z2+45*z-11 ) * ln1mz*ln1mz
- 1./6. * z * (1-z) * ( 34*z2-52*z+17 ) * ln1mz * lnhf1pz
- 1./8. * z * (1-z) * ( 20*z3+106*z2+51*z+25 ) * ln1pz*ln1pz
- 1./6. * (1-z) * ( 90*z4+192*z3+359*z2+228*z+126 ) * ln2 * ln1pz
+ 1./6. * z * (1+z) * (1-z) * ( 20*z2+19+38*z ) 
   * log((1.+z)/2./z2) * log(1.+2.*z)
+ 1./24. * (1-z) * ( 420*z4+1086*z3+1589*z2+987*z+504 ) * ln2*ln2
+ 1./12. * z * (1-z) * ( 51+45*z-2*z2+86*z3 ) * Z2
// q^1:
- 1./6. * ( 97*z5-234*z4+304*z3-170*z2-44*z+81 ) * lnde
+ 1./12./(1+2*z) * ( -1354*z2+346*z3-977*z5+1472*z4-336*z+870*z6+81 )
   * lnz
+ 1./12./(1+2*z) * ( -1299*z3+146*z4+498*z5+552*z-695*z2+264*z6+288 )
   * lnhf1pz
- 1./6. * ( 103*z5-262*z4+359*z3-227*z2-20*z+81 ) * ln1mz
// q^0:
- 1./12. * ( 42 + 222*z - 259*z2 + 38*z3 - 351*z4 + 296*z5 ) ) ;
 return temp;
}

// u -> 1 - z limit of qg:

double qg_u1mz(double y, double z){
 double u,r,t,Z2,z2,z3, ln2,lnz,ln1mz,ln1pz,lnhf1pz, lnde,  temp;
 u = (z+y*(1.-z))/(1.-y*(1.-z));   t = sqrt(z);   r = sqrt(u);
 Z2 = ZETA2;  z2 = z*z;   z3 = z2*z;
 ln2 = log(2.);  lnz = log(z);  ln1mz = log(1.-z);
 ln1pz = log(1.+z);  lnhf1pz = log((1.+z)/2.);
 lnde = log(1./r-t);
 temp = 1./6. * z/(1+z) * ( 
// q^2:
+ lnde * ( ( 11*z3-16*z2+20*z+2 ) * lnde
     - 2 * ( 33*z+23*z3-35*z2+1 ) * lnz
     + ( 21*z3-30*z2+39*z+4 ) * ln1mz
     + 4 * (-3+2*z) * (1+z2) * lnhf1pz )
//
+ 2 * ( 21*z3+2*z2+27*z-2 ) * li2(z)
+ ( -44*z+20+3*z3+14*z2 ) * li2(-z)
- ( 37*z-12+38*z3-52*z2 ) * li2((1.+z)/2.)
//
- 1./4. * ( -2-164*z-348*z2+79*z3 ) * lnz*lnz
+ 1./2. * ( 95*z-12+341*z3-478*z2 ) * lnz * ln1mz
+ ( 31*z3-12*z2-32*z+26 ) * lnz * ln1pz
- ( 53*z-2+120*z3-120*z2 ) * ln1mz*ln1mz
- 3. * z * ( 20+31*z2-34*z ) * lnhf1pz * ln1mz
+ 5. * z * ( 3*z2-4*z+3 ) * ln1pz*ln1pz
- 2. * ( 6*z+3+14*z3-13*z2 ) * ln2 * lnz
+ ( 7*z-12+8*z3-12*z2 ) * ln2 * ln1pz
- (23*z3-32*z2+22*z-12) * ln2*ln2
+ 1./2. * (-16*z-4+29*z3-186*z2) * Z2
// q^1:
- 2. * ( 4*z3-7*z2+5*z+1 ) * lnde
- ( 373*z3-124*z2+23*z-13 ) * lnz
+ ( -26+13*z-252*z2+271*z3 ) * ln1mz
+ ( 115*z3-139*z2-76*z-62 ) * lnhf1pz
// q^0
- (1-z) * ( 117*z2+28*z+54 ) );
 return temp;
}

// z -> 1 limit of qg:

double qg_z1(double y, double z){
  double a,a2,a3,a4,y2,y3,y4,lna,lna2,lny,lny2,ln1my,ln1my2;
  a = 1.-z;  
  a2 = a*a;   a3 = a2*a;   a4 = a2*a2;  
  y2 = y*y;   y3 = y2*y;   y4 = y2*y2; 
  lna = log(a);  lny = log(y);  ln1my = log(1.-y);
  lna2 = lna*lna;   lny2 = lny*lny;  ln1my2 = ln1my*ln1my; 
return 17/12.*ln1my2/y+17/6.*lny/y*ln1my+17/3./y*lna*ln1my-1/2.*ln1my/y
  +a*(1/2.*ZETA2+1/24.*(-119+60*y)/y*ln1my2-1/4.*(-41+22*y)/y*ln1my
    +1/12.*(-119+60*y)/y*lny*ln1my+1/6.*(-119+60*y)/y*lna*ln1my
    +5/3.*lny2-59/12.*lny+25/3.*lna*lny+55/6.*lna2-125/12.*lna+101/24.)
  +a2*(-1/3.*(-11+10*y)*ZETA2+1/16.*(24*y2+119-112*y)/y*ln1my2
    -1/48.*(59*y2-1120*y+1334)/y*ln1my+1/24.*(74*y2-340*y+357)/y*lny*ln1my
    +1/12.*(73*y-119)*(y-3)/y*lna*ln1my+1/6.*(9*y-32)*lny2
    -1/48.*(49*y-990)*lny+1/12.*(-298+73*y)*lna*lny+1/12.*(73*y-318)*lna2
    -1/24.*(54*y-1055)*lna+379/64.*y-2585/96.)
  +a3*(-1/12.*(105+58*y2-141*y)*ZETA2-1/96.*(459+144*y2-504*y+32*y3)/y*ln1my2
    +1/288.*(1774*y3-1575*y2-6846*y+8126)/y*ln1my
    -1/48.*(150*y2+32*y3+459-516*y)/y*lny*ln1my
    -1/24.*(-510*y+459+147*y2+32*y3)/y*lna*ln1my
    +1/12.*(-33*y+6*y2+58)*lny2+1/96.*(-539*y+522*y2-1876)*lny
    +1/24.*(490+8*y2-207*y)*lna*lny-1/24.*(177*y+12*y2-500)*lna2
    +1/144.*(1670*y2-6237-1596*y)*lna+21779/576.-3971/1728.*y2-315/128.*y)
  +a4*(-1/24.*(92*y3-276*y2+303*y-131)*ZETA2
    +1/192.*(-144*y3+136*y4+85-184*y2+152*y)/y*ln1my2
    +1/1152.*(-17312*y3+2333*y4+25938*y2-5828*y-8860)/y*ln1my
    +1/96.*(148*y-198*y2+85-120*y3+128*y4)/y*lny*ln1my
    +1/48.*(132*y4+150*y+85-191*y2-132*y3)/y*lna*ln1my
    +1/24.*(-6-58*y2+27*y3+32*y)*lny2
    +1/1152.*(17694*y+21*y3-9168*y2-4928)*lny
    +1/48.*(-292*y2+172*y3+29*y+50)*lna*lny
    +1/48.*(-81*y-212*y2+152*y3+100)*lna2
    +1/576.*(21816*y-5378-13240*y2+1177*y3)*lna+13177/6912.+6385/3072.*y3
    -141833/4608.*y+91249/6912.*y2);
}

// Fully patched result:

double qg_full(double y, double z){
  // z = 1 patch:
  if (abs(1.-z) < 0.03){ return qg_z1(y,z); }
  // u = 1 patch:
  if (abs(y-0.5) < 0.00005/sqrt(1.-z)){ return qg_u1(z); }
  // u = z patch:
  if (y < 0.00001/sqrt(1.-z)){ return qg_uz(y,z); }
  // u = 1-z patch (different from u = z):
  if (1.-y < 0.00001/sqrt(1.-z)){ return qg_u1mz(y,z); }
  // z = (2*u/(1+u))^2 patch (average values on either side of strip):
  double eps = 1.0e-07;
  if (abs(y-y_sing(z)) < eps){ 
    return 0.5 * ( qg(y_sing(z)-2.*eps,z) 
		 + qg(y_sing(z)+2.*eps,z) );
  }
  if (abs(y-(1.-y_sing(z))) < eps){ 
    return 0.5 * ( qg(1.-y_sing(z)-2.*eps,z) 
		 + qg(1.-y_sing(z)+2.*eps,z) );
  }
  else { return qg(y,z); }
}

// The mu-dependent qg hard term:

double qg_mu(double y, double z, double muFQ){
  double u,t,r,Z2,z2,z3,z4, u2,u3,u4,u5,u6,u7, up12,up13, lnmu, temp;
 u = (z+y*(1.-z))/(1.-y*(1.-z));   t = sqrt(z);   r = sqrt(u);
 Z2 = ZETA2; z2 = z*z;  z3 = z2*z;  z4 = z2*z2;
 u2 = u*u;  u3 = u2*u;  u4 = u2*u2;  u5 = u3*u2; u6 = u3*u3;  u7 = u4*u3;
 up12 = (1.+u)*(1.+u);   up13 = up12*(1.+u);  
 lnmu = 2.*log(muFQ);
//
 temp = 1./6. * z * (1-z) * ( 
- 2 * lnmu*lnmu /(1+z)/(u-z) * ( z*(3-7*z+10*z2) 
    - (1-3*z)*(1-2*z+2*z2)*u - 2*z*(1+z)*u2 )
+ 2 * t/r * (u+z)/(u-z) * (1-2*(1-z)*u) 
  * ( lnmu*lnmu 
    + 2 * lnmu * log((r+t)/(r-t)*(1+t*r)/(1-t*r)/4.) )
//
+ lnmu * (
 + 2 * z * (1+u) * (1-2*z+2*z2)/(1+z)/(u-z)
  * ( 17 * log(1.-z) + 8 * log((1.+z)/2.) )
 - 1/(1+z)/u3/(u-z) * ( - 2*z2*(1+z)*u6
   + 2*z*(1+z)*(z2+z-5)*u5 - 2*(-7*z+2+z4+63*z2-28*z3)*u4
   + 2*(16*z3+6*z-70*z2-9)*u3 + 4*z*(1+z)*(18*z2-19*z+18)*u2
   + 9*z2*(1+z)*(7*z-17)*u + 99*z3*(1+z) )
   * 2. * log(t/(r+t)*(1.+u))
- u/up13/(u-z) * ( 2*(1+z2)*u5 
  - (-31*z-7+2*z3-4*z2)*u4 - (4*z3+23*z2-25-114*z)*u3
  - (-27-130*z+99*z2+26*z3)*u2 - (-23+22*z3-120*z+45*z2)*u
  + 3 + 34*z - 10*z2 - 11*z3 )
   * log(u)
+ 2/(1+z)/u/up13/(u-z) * ( 2*(1+z)*u7 + (1+z)*(29*z+7)*u6 
   + (131*z-17*z2+25+19*z3)*u5 - (381*z2-147*z-45*z3-31+26*z4)*u4
   - (-35-177*z3-137*z+26*z4+547*z2)*u3 
   - (-15-139*z3-29*z+13*z4+402*z2)*u2 + 2*(2+20*z3-5*z-58*z2)*u
   -4*z*(1+z) )
   * log(1.+u)
+  2/(1+z)/up13/(u-z) * ( 2*(-1+z)*(1+z)*(1+z)*u6
   - (1+z)*(2*z3-4*z2+23*z+7)*u5 - (-33*z2+115*z+4*z4+21*z3+23)*u4
   + (371*z2+18*z4-63*z3-145*z-29)*u3 
   + (551*z2+22*z4-181*z3-25-123*z)*u2 
   + (-5-25*z+390*z2+11*z4-147*z3)*u + 2*z*(2+51*z-22*z2) )
   * log(r+t)
 - 4/(1+z)/up13/(u-z) * ( 2*z*(1+z)*u5
   - (z-1-18*z2+10*z3)*u4 + (4*z4-36*z3-12*z+7+61*z2)*u3
   + (-34*z+63*z2+4*z4+7-72*z3)*u2 
   + (41*z2+3-52*z3-22*z+2*z4)*u - z*(5-11*z+14*z2) )
   * log(r-t)
 - 4/u/up13 * ( 2 + (2*z+5)*u + (5+z2+3*z)*u2
   + (2*z2+z+1)*u3 + (1+4*z2+7*z)*u4 + z*(2*z+3)*u5 + z2*u6 )
   * log(1.+t*r)
 - 2/up13/(u-z) * ( 4*z*u5 + 2*(1+2*z2+4*z)*u4
   + 2*(10*z+13*z2+9*z3+12)*u3 + 2*(12+9*z3-9*z2-10*z)*u2
   + (-7*z+11+9*z3-17*z2)*u + 2*z*(1-2*z) )
   * log(1.-t*r)
//
- 1/(1+z)/u2/up13/(u-z) * ( 4*(1+z)*(z2-z+1)*u7
   - z*(-151*z2+4*z3-5*z-60)*u6 - (99*z4+186*z2-37*z-449*z3+69)*u5
   - (-95*z-775*z3-30*z2+316*z4+104)*u4 
   - (-80*z-389*z2-385*z3+111+653*z4)*u3
   - (-519*z2+334*z3+39+822*z4-202*z)*u2 
   - 27*z*(1+z)*(23*z2+2*z-4)*u - 99*z2*(1+z)*(1+2*z) )
+ 2 * t/r * 1/u2/up12/(u-z) * ( 2*(1+z)*u7 - (-1-39*z+2*z2)*u6
   -(3*z-1)*(5*z-27)*u5 - (70*z2-172*z+59)*u4 - (-274*z+82+205*z2)*u3
   - 9*(4-33*z+33*z2)*u2 - 3*z*(87*z-40)*u - 99*z2 )
 ) ) ;
 return temp;
}

// The hard terms induced for muR <> muF:

double qg_muR_hard(double y, double z, double Nf, double muFQ, double muRQ){
  double C_A = 3.;
  return 1/4. * (11/3. * C_A - 2/3. * Nf) * 2. * log(muRQ/muFQ) 
              * (NLO_qg_real(y,z) - NLO_qg_real_soft(y,z));
}

//====================================================================
// "Boost" soft limits:

// mu-dep. part:

double qg_boost_mu(double z, double muFQ){
 double Z2,z2,z3, ln2,lnz,ln1mz, ln1pz,lnhf1pz, lnmu, temp;
 Z2 = ZETA2; ln2 = log(2.);  lnz = log(z); ln1mz = log(1.-z); 
 ln1pz = log(1.+z);  lnhf1pz = log((1.+z)/2.);
 lnmu = 2.*log(muFQ);
 z2 = z*z;  z3 = z2*z;
 temp = - 1./24./z * (
 lnmu*lnmu * (
  - z * (1-2*z+2*z2) * ( 8 * lnhf1pz + 21 * ln1mz )
  + z * (1-56*z+24*z2) * lnz - 1./2. * (12+19*z+56*z2-69*z3) ) 
+ lnmu * ( 
  z * ( (1-2*z+2*z2) * ( 10 * li2((1.+z)/2.) + 63 * ln1mz*ln1mz
      + 42 * lnhf1pz * ln1mz + 16 * ln2 * lnz
      - 5 * (ln1pz*ln1pz - ln2*ln2) )
 - 4 * (5*z2+11*z+8) * li2(z) - 2 * (8*z2-4*z-7) * li2(-z)
  + 3 * (16*z2-42*z-3) * lnz*lnz
  - 2 * (24*z2-20*z+1) * lnz * ln1pz
  - 2 * (92*z2-124*z+35) * lnz * ln1mz
  - 2 * (5*z2-35*z-14) * Z2 )
- ( 12-4*z+92*z2-345*z3 ) * lnz
+ 4 * ( 15+16*z+35*z2-26*z3 ) * lnhf1pz
+ 2 * ( 12+7*z+96*z2-109*z3 ) * ln1mz
+ 50-48*z+143*z2-158*z3 ) ) ;
 return temp;
}

// muR <> muF part:

double qg_boost_muR(double z, double Nf, double muFQ, double muRQ){
  double C_A = 3.;
  return 1/4. * (11/3. * C_A - 2/3. * Nf) * 2. * log(muRQ/muFQ) 
              * NLO_qg_boost(z,muFQ);
}

// now includes mu-dependent terms:

double qg_boost_tot(double z, double Nf, double muFQ, double muRQ){ 
 double Z2,Z3,z2,z3, ln2,lnz,lnz2,lnz3,ln1mz,ln1mz2,ln1mz3,
       ln1pz,lnhf1pz,lnhf1pz2,temp;
 Z2 = ZETA2;  Z3 = ZETA3;
 ln2 = log(2.);  lnz = log(z);  lnz2 = lnz*lnz;  lnz3 = lnz2*lnz;
 ln1mz = log(1.-z);  ln1mz2 = ln1mz*ln1mz;   ln1mz3 = ln1mz2*ln1mz;
 ln1pz = log(1.+z);  lnhf1pz = log((1.+z)/2.);  lnhf1pz2 = lnhf1pz*lnhf1pz;
 z2 = z*z;  z3 = z2*z;
 if (z==0.5) { temp = 0.429960033243594157; }
 else { temp = 1./4./z * (
// q^3
+ z * (
+ 3./2. * (1+2*z+2*z2) * (
   - 3 * li3(-(1.+z)/z) - li3(-1./2./z)
   + li3(1./2./(1+z)) + lnhf1pz * li2(-(1.+z)/z) 
   - 2 * lnz * li2(-(1.+z)/z) - 2 * ln2*ln2 * lnhf1pz 
   + 1./2. * ln2*ln2 * lnz - 7./6. * ln2*ln2*ln2 - ln2 * Z2 )
+ 1./6. * (1-2*z+2*z2) * (
    20 * (li2((1.+z)/2.) - ln2 * lnhf1pz) * ln1mz
  + 11 * lnhf1pz * li2((1.+z)/2.)
  + 52 * ln1mz2 * lnhf1pz - 108 * ln1mz2 * lnz
  - 12 * lnz * li2((1.+z)/2.) + 257./6. * ln1mz3 )
//
+ 1./12. * (2*z2+178*z+1) * li3((1.+z)/2.)
+ 1./12. * (74*z2-22*z-7) * li3((1.-z)/2.)
- 1./2. * (14*z2-2*z+7) * li3(z)
- 1./3. * (2*z2+52*z+1) * li3(1.-z)
+ 1./6. * (34*z2+74*z+17) * li3(-z)
- 1./24. * (2*z2-38*z+1) * li3((1.-z)*(1.+z))
+ 1./12. * (30*z2+22*z-29) * li3((1.-z)/(1.+z))
+ 1./6. * (1-110*z+46*z2) * li3(2.*z/(1.+z))
+ 1./6. * (30*z2-86*z-29) * lnz * li2(-z)
- 1./6. * (59+98*z+30*z2) * ln1mz * li2(z)
- 1./3. * (16*z2-30*z-3) * lnhf1pz * li2(-z)
+ 1./6. * (-78*z+14*z2-15) * lnhf1pz * li2(z)
+ 2./3. * (8*z2+24*z+15) * lnz * li2(z)
- 2./3. * (8*z2-4*z-7) * ln1mz * ( li2(-z) + lnz * ln2 )
//
- 1./36. * (188*z2-406*z-49) * lnz3
+ 1./12. * (98*z2-2*z-17) * lnz2 * lnhf1pz
+ 1./2. * (52*z2+15-78*z) * lnz2 * ln1mz
+ 1./6. * (13-62*z+26*z2) * lnz * lnhf1pz2
- 4./3. * (4-17*z+19*z2) * ln1mz * lnhf1pz * lnz
+ 1./24. * (7-50*z+102*z2) * ln1mz * lnhf1pz2
- 1./72. * (374*z2-378*z+99) * lnhf1pz2*lnhf1pz
+ 1./4. * (22*z2+10*z-11) * ln2 * lnz2
- 1./24. * (254*z2-58*z+83) * ln2 * lnhf1pz2
+ 1./6. * (38*z2-74*z+19) * ln2 * lnz * lnhf1pz
//
- 1./12. * (114*z2+110*z+35) * Z2 * lnhf1pz
- 1./2. * (10*z2-50*z-17) * Z2 * ln1mz
+ (4*z+1) * (3*z-5) * Z2 * lnz 
- 1./24. * (70*z2+70*z-53) * Z3 )
// q^2
- 3./2. * z * (1+2*z+2*z2) * ( 4. * li2(-z) 
  - lnhf1pz * ln1pz + 2. * ln1pz * lnz - lnz2 + 2. * Z2 )
- 1./6. * (108*z2+12+6*z+119*z3) * li2(z)
+ 1./6. * (-18*z2-48*z+143*z3-96) * li2(-z)
- 1./6. * (13*z3+36*z2+69*z+48) * li2((1.+z)/2.)
+ 1./6. * (-54*z2-96+107*z3-66*z) * lnz * ln2
- 1./6. * (-60*z-48+5*z3-18*z2) * lnhf1pz * ln2
- 1./6. * (-235*z2+261*z3+10*z-24) * ln1mz2
- 1./24. * (1606*z3-172*z2+97*z-24) * lnz2
- 1./6. * (-244*z2+221*z3-59*z-72) * lnhf1pz * ln1mz
- 1./12. * (-124*z2-108+111*z3-115*z) * lnhf1pz2
+ 1./6. * (627*z3-348*z2+26*z-36) * lnz * ln1mz
+ 1./6. * (299*z3-174*z2-158*z-156) * lnhf1pz * lnz
+ 1./12. * z * (543*z2+160*z+112) * Z2
// q^1
+ 1./6. * (-110*z2+100*z3+149*z-50) * lnz
- 1./12. * (556*z3-491*z2+192*z-200) * ln1mz
+ 1./3. * (47*z3+144*z2-11*z+20) * lnhf1pz
// q^0
+ 1./36. * ( 268 - 249 * z + 81 * z2 - 214 * z3 ) );
 }
 return temp + qg_boost_mu(z,muFQ) + qg_boost_muR(z,Nf,muFQ,muRQ);
}

//====================================================================
// qg "Real" soft limits:

// qg DD1(n,y) terms, including conversion factor (for y = 0):
// auxiliary function (now includes mu-dep. terms):

double qg_mu_aux(double y, double z, double muFQ){
 double z2,z3, lnmu;
 z2 = z*z;    z3 = z2*z;  
 lnmu = 2.*log(muFQ);
 return - 1./6. * z/(1+z) * (
 24 * z * (1-2*z+2*z2) * lnmu * DD1(1.,y)
+ ( - 8 * z * (1-2*z+2*z2) * lnmu*lnmu
  + ( 2 * z * (1-2*z+2*z2) * ( 8 * log((1.+z)/2.) + 29 * log(1.-z) )
    - 2 * z * (5-64*z+32*z2) * log(z) + 12+7*z+96*z2-109*z3 ) * lnmu )
 * DD1(0.,y) ) ;
}

// The qg soft terms induced for muF <> muR:

double qg_muR_soft(double y, double z, double Nf, double muFQ, double muRQ){
  double C_A = 3.;
  return 1/4. * (11/3. * C_A - 2/3. * Nf) * 2. * log(muRQ/muFQ) 
              * NLO_qg_real_soft(y,z);
}

double qg_soft_aux(double y, double z, double muFQ){
 double Z2,z2,z3, ln2,lnz,ln1mz,lnhf1pz;
 Z2 = ZETA2;  z2 = z*z;   z3 = z2*z;
 ln2 = log(2.);  lnz = log(z);  ln1mz = log(1.-z);
 lnhf1pz = log((1.+z)/2.);
 return 1./3. * z/(1+z) * ( 6 * z * (1-2*z+2*z2) * DD1(2.,y)
    + ( z * (1-2*z+2*z2) * ( 8 * lnhf1pz + 29 * ln1mz )
      - z * (5-64*z+32*z2) * lnz - 133./2. * z3 + 60 * z2 - 5./2. * z + 6 )
                                        * DD1(1.,y)
    + ( 1./2. * z * (1-2*z+2*z2) * ( 
            10 * (li2((1.+z)/2.) - ln2 * lnhf1pz) 
          + 42 * ln1mz * lnhf1pz + 63 * ln1mz*ln1mz - 5 * lnhf1pz*lnhf1pz )
       + z * (7+4*z-8*z2) * ( li2(-z) + ln2 * lnz )
       - 2 * z * (5*z2+11*z+8) * li2(z)
       + 3./2. * z * (16*z2-42*z-3) * lnz*lnz
       - z * (24*z2-20*z+1) * lnz * lnhf1pz
       - z  * (92*z2-124*z+35) * lnz * ln1mz 
       - z * (5*z2-35*z-14) * Z2
       + 1./2. * (369*z3-116*z2+16*z-12) * lnz
       - (133*z3-120*z2+5*z-12) * ln1mz
       - 2 * (-35*z2+26*z3-16*z-15) * lnhf1pz
	- 67*z3 + 119./2.*z2 - 24*z + 25 ) * DD1(0.,y) )
   + qg_mu_aux(y,z,muFQ);
}

double qg_real_soft(double y, double z, double Nf, double muFQ, double muRQ){
 return real_conv(0.,z) * qg_soft_aux(y,z,muFQ)
        + qg_muR_soft(y,z,Nf,muFQ,muRQ) ;
}

// A remainder term, from "soft", treated as "hard" (no subtraction):

double qg_soft_to_hard(double y, double z, double muFQ){
 return ( real_conv(y,z) - real_conv(0.,z) ) * qg_soft_aux(y,z,muFQ);
}

//====================================================================
// The full q-\bar{q} real "hard" term, including conversion factor,
// and adding in all other terms multiplying "qg_lumi_ys":

double qg_real_hard(double y, double z, double Nf, double muFQ, double muRQ){
  return real_conv(y,z) * ( qg_full(y,z) + qg_mu(y,z,muFQ) ) 
      + qg_muR_hard(y,z,Nf,muFQ,muRQ) 
      + qg_soft_to_hard(y,z,muFQ) + qg_real_soft(y,z,Nf,muFQ,muRQ) ; 
}
